@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,18 +34,18 @@ typedef unsigned envid_t;
 #endif
 
 /*
- * VE management ioctls
+ * CT management ioctls
  */
 
 struct vzctl_old_env_create {
 	envid_t veid;
 	unsigned flags;
-#define VE_CREATE 	1	/* Create VE, VE_ENTER added automatically */
+#define VE_CREATE	1	/* Create CT, VE_ENTER added automatically */
 #define VE_EXCLUSIVE	2	/* Fail if exists */
-#define VE_ENTER	4	/* Enter existing VE */
-#define VE_TEST		8	/* Test if VE exists */
-#define VE_LOCK		16	/* Do not allow entering created VE */
-#define VE_SKIPLOCK	32	/* Allow entering embrion VE */
+#define VE_ENTER	4	/* Enter existing CT */
+#define VE_TEST		8	/* Test if CT exists */
+#define VE_LOCK		16	/* Do not allow entering created CT */
+#define VE_SKIPLOCK	32	/* Allow entering "embryo" CT */
 	__u32 addr;
 };
 
@@ -102,6 +102,7 @@ struct vzctl_ve_meminfo {
 #define VE_IP_TARGET_REDIRECT_MOD	(1U<<23)
 #define VE_IP_MATCH_OWNER_MOD		(1U<<24)
 #define VE_IP_MATCH_MAC_MOD		(1U<<25)
+#define VE_IP_MATCH_RECENT_MOD		(1U<<26)
 
 /* these masks represent modules with their dependences */
 #define VE_IP_IPTABLES		(VE_IP_IPTABLES_MOD)
@@ -153,13 +154,15 @@ struct vzctl_ve_meminfo {
 					| VE_IP_IPTABLES)
 #define VE_IP_MATCH_OWNER	(VE_IP_MATCH_OWNER_MOD	\
 					| VE_IP_IPTABLES)
+#define VE_IP_MATCH_RECENT	(VE_IP_MATCH_RECENT_MOD	\
+					| VE_IP_IPTABLES)
 
 /* safe iptables mask to be used by default */
 #define VE_IP_DEFAULT					\
 	(VE_IP_IPTABLES |				\
 	VE_IP_FILTER | VE_IP_MANGLE |			\
 	VE_IP_MATCH_LIMIT | VE_IP_MATCH_MULTIPORT |	\
-	VE_IP_MATCH_TOS | VE_IP_TARGET_REJECT | 	\
+	VE_IP_MATCH_TOS | VE_IP_TARGET_REJECT |		\
 	VE_IP_TARGET_TCPMSS | VE_IP_MATCH_TCPMSS |	\
 	VE_IP_MATCH_TTL | VE_IP_MATCH_LENGTH)
 
@@ -200,6 +203,12 @@ struct env_create_param3 {
 #define VE_FEATURE_SYSFS	(1ULL << 0)
 #define VE_FEATURE_NFS		(1ULL << 1)
 #define VE_FEATURE_DEF_PERMS	(1ULL << 2)
+#define VE_FEATURE_SIT		(1ULL << 3)
+#define VE_FEATURE_IPIP		(1ULL << 4)
+#define VE_FEATURE_PPP		(1ULL << 5)
+#define VE_FEATURE_IPGRE	(1ULL << 6)
+#define VE_FEATURE_BRIDGE	(1ULL << 7)
+#define VE_FEATURE_NFSD		(1ULL << 8)
 
 #define VE_FEATURES_OLD		(VE_FEATURE_SYSFS)
 #define VE_FEATURES_DEF		(VE_FEATURE_SYSFS | \
@@ -254,7 +263,7 @@ struct vzctl_cpustatctl {
 					struct vzctl_env_create_data)
 #define VZCTL_VE_NETDEV		_IOW(VZCTLTYPE, 11,			\
 					struct vzctl_ve_netdev)
-#define VZCTL_VE_MEMINFO	_IOW(VZCTLTYPE, 13,                     \
+#define VZCTL_VE_MEMINFO	_IOW(VZCTLTYPE, 13,			\
 					struct vzctl_ve_meminfo)
 
 #ifdef __KERNEL__
@@ -283,7 +292,7 @@ struct compat_vzctl_env_create_data {
 					struct compat_vzctl_env_create_data)
 #define VZCTL_COMPAT_VE_NETDEV	_IOW(VZCTLTYPE, 11,			\
 					struct compat_vzctl_ve_netdev)
-#define VZCTL_COMPAT_VE_MEMINFO	_IOW(VZCTLTYPE, 13,                     \
+#define VZCTL_COMPAT_VE_MEMINFO	_IOW(VZCTLTYPE, 13,			\
 					struct compat_vzctl_ve_meminfo)
 #endif
 #endif
