@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,8 +21,6 @@
 #include <stdio.h>
 #include "types.h"
 
-#define	LOG_DATA	log_param g_log = {NULL, 0, 1, 0, 0, "", 0};
-
 /** Data structure for logging.
  */
 typedef struct {
@@ -41,7 +39,8 @@ typedef struct {
  * @param err_num		errno
  * @param format		fprintf format.
  */
-void logger(int log_level, int err_num, const char *format, ...);
+void logger(int log_level, int err_num, const char *format, ...)
+	__attribute__ ((__format__ (__printf__, 3, 4)));
 
 /** Change/set log file.
  *
@@ -53,14 +52,14 @@ int set_log_file(char *file);
 /** Initialize logging.
  *
  * @param file		log file.
- * @param veid		VE id.
+ * @param veid		CT ID.
  * @param enable	enable/disable logging.
  * @param level		maximum logging level.
  * @param quiet		skip logging to stdout.
  * @param progname	program name.
  */
 int init_log(char *file, envid_t veid, int enable, int level, int quiet,
-	char *progname);
+	const char *progname);
 
 /** Close logging.
  */
