@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #define QUOTA_DROP		1
 #define QUOTA_STAT		2
 #define QUOTA_STAT2		3
-#define QUOTA_MARKDURTY		9
+#define QUOTA_MARKDIRTY		9
 #define QUOTA_SHOW		5
 
 #define QUOTA_EXPTIME		259200
@@ -37,55 +37,21 @@
 /** Data structure for disk quota parameters.
  */
 typedef struct {
-	int enable;			/**< quota enable yes/no. */
-	unsigned long *diskspace;	/**< disk block limit. */
+	int enable;			/**< is quota enabled (YES or NO). */
+	unsigned long *diskspace;	/**< disk blocks limit. */
 	unsigned long *diskinodes;	/**< disk inodes limit. */
-	unsigned long *exptime;		/**< quot aexpiration time. */
-	unsigned long *ugidlimit;	/**< userqroup quota limit. */
+	unsigned long *exptime;		/**< quota expiration time. */
+	unsigned long *ugidlimit;	/**< user/group quota limit. */
 } dq_param;
 
-/** Setup disk quota limits.
- *
- * @param veid		CT ID.
- * @param dq		disk quota parameters.
- * @return		0 on success.
- */
 int vps_set_quota(envid_t veid, dq_param *dq);
-
-/** Turn disk quota on.
- *
- * @param veid		CT ID.
- * @param private	CT private area path.
- * @param dq		disk quota parameters.
- * @return		0 on success.
- */
 int vps_quotaon(envid_t veid, char *private, dq_param *dq);
-
-/** Turn disk quota off.
- *
- * @param veid		CT ID.
- * @param dq		disk quota parameters.
- * @return		0 on success.
- */
 int vps_quotaoff(envid_t veid, dq_param *dq);
 
-/** Disk quota managment wraper.
- *
- * @param veid		CT ID.
- * @param cmd		quota commands (QUOTA_MARKDURTY QUOTA_DROP QUOTA_STAT)
- * @return		0 on success.
- */
 int quota_ctl(envid_t veid, int cmd);
-
-/** Turn quota off.
- *
- * @param veid		CT ID.
- * @param force		forcebly turn quota off.
- * @return		0 on success.
- */
 int quota_off(envid_t veid, int force);
 int quota_on(envid_t veid, char *private, dq_param *param);
 int quota_set(envid_t veid, char *private, dq_param *param);
 int quota_init(envid_t veid, char *private, dq_param *param);
-void quouta_inc(dq_param *param, int delta);
+void quota_inc(dq_param *param, int delta);
 #endif
