@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ typedef struct {
 	 * in this mask kernel sets to default values.
 	 */
 	unsigned long long features_mask;
-        unsigned long long features_known;
+	unsigned long long features_known;
 } env_param;
 
 typedef struct {
@@ -51,17 +51,24 @@ typedef struct {
 	list_head_t nameserver;
 	list_head_t searchdomain;
 	char *hostname;
+	char *description;
+	int onboot;
+	unsigned long *bootorder;
 	int wait;
 } misc_param;
 
 struct mod_action;
 
+typedef struct {
+	int ioprio;
+} io_param;
+
 typedef struct name_param {
 	int veid;
 	char *name;
 } name_param;
 
-/** Data structure for VE resources.
+/** Data structure for CT resources.
  */
 typedef struct vps_res {
 	fs_param fs;		/**< file system parameters. */
@@ -78,25 +85,26 @@ typedef struct vps_res {
 	veth_param veth;	/**< veth parameters */
 	misc_param misc;
 	name_param name;
+	io_param io;
 } vps_res;
 
 enum {
-	APPCONF_MAP_NAME    = 0x01,
+	APPCONF_MAP_NAME = 0x01,
 };
 
 typedef struct {
-        int save;
-        int fast_kill;
-        int skip_lock;
-        int skip_setup;
-        int start_disabled;
-        int start_force;
-        int setmode;
-        int onboot;
-        char *config;
-        char *origin_sample;
-        char *lockdir;
-        char *apply_cfg;
+	int save;
+	int save_force;
+	int fast_kill;
+	int skip_lock;
+	int skip_setup;
+	int start_disabled;
+	int start_force;
+	int setmode;
+	char *config;
+	char *origin_sample;
+	char *lockdir;
+	char *apply_cfg;
 	int apply_cfg_map;
 	int reset_ub;
 } vps_opt;
@@ -118,14 +126,14 @@ struct vps_param {
 };
 typedef struct vps_param vps_param;
 
-/** Setting VE resources.
+/** Setting CT resources.
  *
- * @param h		VE handler.
- * @param veid		VE id.
+ * @param h		CT handler.
+ * @param veid		CT ID.
  * @param actions	distribution action scripts.
- * @param fs		VE FS parameters
- * @param param		VE parameters.
- * @param vps_state	VE state.
+ * @param fs		CT FS parameters
+ * @param param		CT parameters.
+ * @param vps_state	CT state.
  * @param action	external modules.
  * @return		0 on success.
  */
