@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -97,6 +97,9 @@
 #ifndef UB_IPTENTRIES
 #define UB_IPTENTRIES	23	/* Number of iptables rules */
 #endif
+#ifndef UB_SWAPPAGES
+#define UB_SWAPPAGES	24
+#endif
 #define UB_DUMMY	255
 
 /** Data structure for UBC parameter.
@@ -130,6 +133,7 @@ struct ub_struct {
 	unsigned long *dcachesize;
 	unsigned long *numiptent;
 	unsigned long *avnumproc;
+	unsigned long *swappages;
 };
 typedef struct ub_struct ub_param;
 
@@ -166,7 +170,7 @@ int add_ub_param(ub_param *ub, ub_res *res);
  */
 int vps_read_ubc(envid_t veid, ub_param *ub);
 int get_ub_resid(char *name);
-const char *get_ub_name(int res_id);
+const char *get_ub_name(unsigned int res_id);
 void add_ub_limit(struct ub_struct *ub, int res_id, unsigned long *limit);
 void free_ub_param(ub_param *ub);
 void merge_ub(ub_param *dst, ub_param *src);
