@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -65,8 +65,8 @@ typedef struct {
  */
 struct vps_param;
 int vps_setup_veth(vps_handler *h, envid_t veid, dist_actions *actions,
-	char *root, veth_param *veth_add, veth_param *veth_del, int state,
-	int skip);
+	const char *root, veth_param *veth_add, veth_param *veth_del,
+	int state, int skip);
 int add_veth_param(veth_param *list, veth_dev *dev);
 void free_veth(list_head_t *head);
 void free_veth_param(veth_param *dev);
