@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,13 +34,15 @@
 enum {
 	VE_RUNNING,
 	VE_STOPPED,
-	VE_MOUNTED
+	VE_MOUNTED,
+	VE_SUSPENDED
 };
 
 char *ve_status[]= {
 	"running",
 	"stopped",
-	"mounted"
+	"mounted",
+	"suspended"
 };
 
 struct Cubc {
@@ -64,6 +66,7 @@ struct Cubc {
 	unsigned long dcachesize[5];
 	unsigned long numfile[5];
 	unsigned long numiptent[5];
+	unsigned long swappages[5];
 };
 
 struct Cquota {
@@ -79,11 +82,16 @@ struct Ccpu {
 	unsigned long limit[2];		// 0-limit, 1-units
 };
 
+struct Cio {
+	int ioprio;
+};
+
 struct Cveinfo {
 	int veid;
 	char *hostname;
 	char *name;
 	char *description;
+	char *ostemplate;
 	char *ip;
 	char *ve_private;
 	char *ve_root;
@@ -91,8 +99,11 @@ struct Cveinfo {
 	struct Cquota *quota;
 	struct Cla *la;
 	struct Ccpu *cpu;
+	struct Cio io;
 	int status;
 	int hide;
+	int onboot;
+	unsigned long *bootorder;
 };
 
 #define RES_NONE	0
@@ -102,8 +113,6 @@ struct Cveinfo {
 #define RES_IP		4
 #define RES_LA		5
 #define RES_CPU		6
-#define RES_NAME	7
-#define RES_DESCRIPTION	8
 
 struct Cfield {
 	char *name;
