@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,6 +15,9 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -83,6 +86,7 @@ static vps_config config[] = {
 {"NUMIPTENT",	NULL, PARAM_NUMIPTENT},
 {"IPTENTRIES",	"NUMIPTENT", -1},
 {"AVNUMPROC",	NULL, PARAM_AVNUMPROC},
+{"SWAPPAGES",	NULL, PARAM_SWAPPAGES},
 /*	Capability */
 {"CAPABILITY",	NULL, PARAM_CAP},
 /*	Network	*/
@@ -131,14 +135,16 @@ static vps_config config[] = {
 
 {"FEATURES",	NULL, PARAM_FEATURES},
 {"IOPRIO",	NULL, PARAM_IOPRIO},
+{"BOOTORDER",	NULL, PARAM_BOOTORDER},
+
 {NULL		,NULL, -1}
 };
 
 static struct option set_opt[] = {
 {"save",	no_argument, NULL, PARAM_SAVE},
+{"force",	no_argument, NULL, PARAM_FORCE},
 {"applyconfig",	required_argument, NULL, PARAM_APPLYCONFIG},
 {"applyconfig_map",	required_argument, NULL, PARAM_APPLYCONFIG_MAP},
-{"config",	required_argument, NULL, PARAM_CONFIG},
 {"reset_ub",	no_argument, NULL, PARAM_RESET_UB},
 {"iptables",	required_argument, NULL, PARAM_IPTABLES},
 /*	UB	*/
@@ -149,7 +155,7 @@ static struct option set_opt[] = {
 {"numproc",	required_argument, NULL, PARAM_NUMPROC},
 {"physpages",	required_argument, NULL, PARAM_PHYSPAGES},
 {"vmguarpages",	required_argument, NULL, PARAM_VMGUARPAGES},
-{"oomguarpagess",required_argument, NULL, PARAM_OOMGUARPAGES},
+{"oomguarpages",required_argument, NULL, PARAM_OOMGUARPAGES},
 {"numtcpsock",	required_argument, NULL, PARAM_NUMTCPSOCK},
 {"numflock",	required_argument, NULL, PARAM_NUMFLOCK},
 {"numpty",	required_argument, NULL, PARAM_NUMPTY},
@@ -158,11 +164,12 @@ static struct option set_opt[] = {
 {"tcprcvbuf",	required_argument, NULL, PARAM_TCPRCVBUF},
 {"othersockbuf",required_argument, NULL, PARAM_OTHERSOCKBUF},
 {"dgramrcvbuf",	required_argument, NULL, PARAM_DGRAMRCVBUF},
-{"numothersockk",required_argument, NULL, PARAM_NUMOTHERSOCK},
+{"numothersock",required_argument, NULL, PARAM_NUMOTHERSOCK},
 {"numfile",	required_argument, NULL, PARAM_NUMFILE},
 {"dcachesize",	required_argument, NULL, PARAM_DCACHESIZE},
 {"numiptent",	required_argument, NULL, PARAM_NUMIPTENT},
 {"avnumproc",	required_argument, NULL, PARAM_AVNUMPROC},
+{"swappages",	required_argument, NULL, PARAM_SWAPPAGES},
 /*	Capability */
 {"capability",	required_argument, NULL, PARAM_CAP},
 /*	Network	*/
@@ -221,6 +228,7 @@ static struct option set_opt[] = {
 {"features",	required_argument, NULL, PARAM_FEATURES},
 {"ioprio",	required_argument, NULL, PARAM_IOPRIO},
 {"description",	required_argument, NULL, PARAM_DESCRIPTION},
+{"bootorder",	required_argument, NULL, PARAM_BOOTORDER},
 
 {NULL, 0, NULL, 0}
 };
@@ -297,31 +305,28 @@ static int parse_setmode(vps_param *vps_p, const char *val)
 	return 0;
 }
 
-int conf_parse_strlist(list_head_t *list, const char *val, int checkdup)
+static int conf_parse_strlist(list_head_t *list, const char *val)
 {
 	if (add_str2list(list, val))
 		return ERR_NOMEM;
 	return 0;
 }
 
-int conf_parse_str(char **dst, const char *val, int checkdup)
+static int conf_parse_str(char **dst, const char *val)
 {
-	if (*dst != NULL) {
-		if (checkdup)
-			return ERR_DUP;
-		free(*dst);
-	}
+	if (*dst)
+		return ERR_DUP;
 	*dst = strdup(val);
 	if (*dst == NULL)
 		return ERR_NOMEM;
 	return 0;
 }
 
-int conf_parse_yesno(int *dst, const char *val, int checkdup)
+static int conf_parse_yesno(int *dst, const char *val)
 {
 	int ret;
 
-	if (*dst && checkdup)
+	if (*dst)
 		return ERR_DUP;
 	if ((ret = yesno2id(val)) < 0)
 		return ERR_INVAL;
@@ -329,11 +334,26 @@ int conf_parse_yesno(int *dst, const char *val, int checkdup)
 	return 0;
 }
 
-int conf_store_strlist(list_head_t *conf, char *name, list_head_t *val)
+static int conf_parse_ulong(unsigned long **dst, const char *valstr)
+{
+	unsigned long val;
+	if (*dst != NULL)
+		return ERR_DUP;
+	if (parse_ul(valstr, &val) != 0)
+		return ERR_INVAL;
+	*dst = malloc(sizeof(unsigned long));
+	if (*dst == NULL)
+		return ERR_NOMEM;
+	**dst = val;
+	return 0;
+}
+
+static int conf_store_strlist(list_head_t *conf, char *name, list_head_t *val,
+		int allow_empty)
 {
 	char *str;
 
-	if (list_empty(val))
+	if (list_empty(val) && !allow_empty)
 		return 0;
 	if ((str = list2str_c(name, '"', val)) == NULL)
 		return ERR_NOMEM;;
@@ -344,7 +364,7 @@ int conf_store_strlist(list_head_t *conf, char *name, list_head_t *val)
 	return 0;
 }
 
-int conf_store_str(list_head_t *conf, char *name, const char *val)
+static int conf_store_str(list_head_t *conf, char *name, const char *val)
 {
 	char *buf;
 	int len;
@@ -361,46 +381,43 @@ int conf_store_str(list_head_t *conf, char *name, const char *val)
 	return 0;
 }
 
-int conf_store_yesno(list_head_t *conf, char *name, int val)
+static int conf_store_yesno(list_head_t *conf, char *name, int val)
 {
-	char *buf;
-	int len;
-
 	if (!val)
 		return 0;
-	len = strlen(name) + 6;
-	buf = malloc(len + 1);
-	if (buf == NULL)
-		return ERR_NOMEM;
-	sprintf(buf, "%s=\"%s\"", name,	val == YES ? "yes" : "no");
-	if (add_str_param2(conf, buf))
-		return ERR_NOMEM;
-	return 0;
+	return conf_store_str(conf, name, val == YES ? "yes" : "no");
+}
+
+static int conf_store_ulong(list_head_t *conf, char *name, unsigned long *val)
+{
+	char buf[] = "18446744073709551615"; /* ULONG_MAX on 64 bit */
+
+	if (val == NULL)
+		return 0;
+
+	snprintf(buf, sizeof(buf), "%lu", *val);
+	return conf_store_str(conf, name, buf);
 }
 
 /******************** Features *************************/
 static int parse_features(env_param *env, char *val)
 {
-	int ret;
+	int ret = 0;
 	char *token;
 	struct feature_s *feat;
 
-	if ((token = strtok(val, "\t ")) == NULL)
-		return 0;
-
-	ret = 0;
-	do {
+	for_each_strtok(token, val, "\t ") {
 		feat = find_feature(token);
-		if (feat == NULL) {
+		if (!feat) {
 			logger(0, 0, "Warning: Unknown feature: %s", token);
 			ret = ERR_INVAL_SKIP;
 			continue;
 		}
-
 		if (feat->on)
 			env->features_mask |= feat->mask;
 		env->features_known |= feat->mask;
-	} while ((token = strtok(NULL, "\t ")) != NULL);
+	}
+
 	return ret;
 }
 
@@ -420,8 +437,10 @@ int parse_ioprio(int id, io_param *io, char *val)
 {
 	if (parse_int(val, &io->ioprio))
 		return ERR_INVAL;
-	if (io->ioprio < VE_IOPRIO_MIN || io->ioprio > VE_IOPRIO_MAX)
+	if (io->ioprio < VE_IOPRIO_MIN || io->ioprio > VE_IOPRIO_MAX) {
+		io->ioprio = -1;
 		return ERR_INVAL;
+	}
 	return 0;
 }
 
@@ -447,20 +466,19 @@ static int parse_iptables(env_param *env, char *val)
 {
 	char *token;
 	struct iptables_s *ipt;
-	int ret;
+	int ret = 0;
 
-	if ((token = strtok(val, "\t ")) == NULL)
-		return 0;
-	ret = 0;
-	do {
-		if ((ipt = find_ipt(token)) == NULL) {
+	for_each_strtok(token, val, "\t ") {
+		ipt = find_ipt(token);
+		if (!ipt) {
 			logger(-1, 0, "Warning: Unknown iptable module: %s,"
 				" skipped", token);
 			ret = ERR_INVAL_SKIP;
 			continue;
 		}
-		env->ipt_mask |= (unsigned long) ipt->mask;
-	} while ((token = strtok(NULL, "\t ")));
+		env->ipt_mask |= (unsigned long)ipt->mask;
+	}
+
 	return ret;
 }
 
@@ -685,6 +703,7 @@ if (ub->res != NULL) {							\
 	ADD_UB_PARAM(dcachesize, PARAM_DCACHESIZE)
 	ADD_UB_PARAM(numiptent, PARAM_NUMIPTENT)
 	ADD_UB_PARAM(avnumproc, PARAM_AVNUMPROC)
+	ADD_UB_PARAM(swappages, PARAM_SWAPPAGES)
 #undef ADD_UB_PARAM
 
 	return 0;
@@ -694,14 +713,12 @@ if (ub->res != NULL) {							\
 
 static int parse_cap(char *str, cap_param *cap)
 {
-	int len;
+	unsigned int len;
 	char *p, *token;
 	char cap_nm[128];
 	unsigned long *mask;
 
-	if ((token = strtok(str, "\t ")) == NULL)
-		return 0;
-	do {
+	for_each_strtok(token, str, "\t ") {
 		if ((p = strrchr(token, ':')) == NULL) {
 			logger(-1, 0, "Invalid syntaxes in %s:"
 				" capname:on|off", token);
@@ -724,7 +741,7 @@ static int parse_cap(char *str, cap_param *cap)
 			logger(-1, 0, "Capability %s is unknown", cap_nm);
 			return ERR_INVAL;
 		}
-	} while ((token = strtok(NULL, " ")));
+	}
 
 	return 0;
 }
@@ -782,11 +799,10 @@ static int parse_ip(vps_param *vps_p, char *val, int id)
 	else
 		return 0;
 
-	if ((token = strtok(val, "\t ")) == NULL)
-		return 0;
-	do {
+	for_each_strtok(token, val, "\t ") {
 		if (id == PARAM_IP_DEL && !strcmp(token, "all")) {
 			vps_p->res.net.delall = YES;
+			vps_p->del_res.net.delall = YES;
 			continue;
 		}
 		if (!strcmp(token, "0.0.0.0"))
@@ -805,7 +821,7 @@ static int parse_ip(vps_param *vps_p, char *val, int id)
 			return ERR_INVAL;
 		if (!find_ip(&net->ip, dst))
 			add_str_param(&net->ip, dst);
-	} while ((token = strtok(NULL, " ")));
+	}
 
 	return 0;
 }
@@ -814,7 +830,7 @@ static int store_ip(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
 	list_head_t ip;
-	char *str;
+	int ret;
 
 	list_head_init(&ip);
 	if (conf->id != PARAM_IP_ADD)
@@ -827,14 +843,10 @@ static int store_ip(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	}
 	merge_str_list(vps_p->res.net.delall, &old_p->res.net.ip,
 		&vps_p->res.net.ip, &vps_p->del_res.net.ip, &ip);
-	str = list2str_c(conf->name, '"', &ip);
+	ret = conf_store_strlist(conf_h, conf->name, &ip, 1);
 	free_str_param(&ip);
-	if (str == NULL)
-		return -1;
-	add_str_param(conf_h, str);
-	free(str);
 
-	return 0;
+	return ret;
 }
 
 static int check_netdev(const char *devname)
@@ -856,13 +868,11 @@ int add_netdev(net_param *net, char *val)
 {
 	char *token;
 
-	if ((token = strtok(val, "\t ")) == NULL)
-		return 0;
-	do {
+	for_each_strtok(token, val, "\t ") {
 		if (check_netdev(token))
 			return ERR_INVAL;
 		add_str_param(&net->dev, token);
-	} while ((token = strtok(NULL, "\t ")));
+	}
 
 	return 0;
 }
@@ -871,7 +881,7 @@ static int store_netdev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
 	list_head_t dev;
-	char *str;
+	int ret;
 
 	list_head_init(&dev);
 	if (conf->id != PARAM_NETDEV_ADD)
@@ -883,14 +893,10 @@ static int store_netdev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	}
 	merge_str_list(0, &old_p->res.net.dev, &vps_p->res.net.dev,
 		 &vps_p->del_res.net.dev, &dev);
-	str = list2str_c(conf->name, '"', &dev);
+	ret = conf_store_strlist(conf_h, conf->name, &dev, 1);
 	free_str_param(&dev);
-	if (str == NULL)
-		return -1;
-	add_str_param(conf_h, str);
-	free(str);
 
-	return 0;
+	return ret;
 }
 
 /********************* fs *******************************/
@@ -971,18 +977,10 @@ static int store_dq(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 		add_str_param(conf_h, buf);
 		break;
 	case PARAM_QUOTATIME:
-		if (param->exptime == NULL)
-			break;
-		snprintf(buf, sizeof(buf), "%s=\"%lu\"", conf->name,
-			 param->exptime[0]);
-		add_str_param(conf_h, buf);
+		conf_store_ulong(conf_h, conf->name, param->exptime);
 		break;
 	case PARAM_QUOTAUGIDLIMIT:
-		if (param->ugidlimit == NULL)
-			break;
-		snprintf(buf, sizeof(buf), "%s=\"%lu\"", conf->name,
-			param->ugidlimit[0]);
-		add_str_param(conf_h, buf);
+		conf_store_ulong(conf_h, conf->name, param->ugidlimit);
 		break;
 	}
 	return 0;
@@ -1049,14 +1047,12 @@ static int parse_dev(vps_param *vps_p, char *val)
 	char *token;
 	dev_res dev;
 
-	if ((token = strtok(val, " ")) == NULL)
-		return 0;
-	do {
+	for_each_strtok(token, val, " ") {
 		if (parse_devices_str(token, &dev))
 			return ERR_INVAL;
 		if (add_dev_param(&vps_p->res.dev, &dev))
 			return ERR_NOMEM;
-	} while ((token = strtok(NULL, " ")));
+	}
 
 	return 0;
 }
@@ -1112,7 +1108,7 @@ static int store_dev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 static int parse_devnodes_str(const char *str, dev_res *dev)
 {
 	char *ch;
-	int len;
+	unsigned int len;
 	char buf[64];
 	struct stat st;
 
@@ -1149,14 +1145,12 @@ static int parse_devnodes(vps_param *vps_p, char *val)
 	char *token;
 	dev_res dev;
 
-	if ((token = strtok(val, " ")) == NULL)
-		return 0;
-	do {
+	for_each_strtok(token, val, " ") {
 		if (parse_devnodes_str(token, &dev))
 			return ERR_INVAL;
 		if (add_dev_param(&vps_p->res.dev, &dev))
 			return ERR_NOMEM;
-	} while ((token = strtok(NULL, " ")));
+	}
 
 	return 0;
 }
@@ -1209,7 +1203,11 @@ static int store_misc(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	ret = 0;
 	switch (conf->id) {
 	case PARAM_ONBOOT:
-		ret = conf_store_yesno(conf_h, conf->name, vps_p->opt.onboot);
+		ret = conf_store_yesno(conf_h, conf->name, misc->onboot);
+		break;
+	case PARAM_BOOTORDER:
+		ret = conf_store_ulong(conf_h, conf->name,
+				misc->bootorder);
 		break;
 	case PARAM_DISABLED:
 		ret = conf_store_yesno(conf_h, conf->name,
@@ -1223,14 +1221,15 @@ static int store_misc(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 		ret = conf_store_str(conf_h, conf->name, misc->hostname);
 		break;
 	case PARAM_NAMESERVER:
-		ret = conf_store_strlist(conf_h, conf->name, &misc->nameserver);
+		ret = conf_store_strlist(conf_h, conf->name,
+				&misc->nameserver, 0);
 		break;
 	case PARAM_DESCRIPTION:
 		ret = conf_store_str(conf_h, conf->name, misc->description);
 		break;
 	case PARAM_SEARCHDOMAIN:
 		ret = conf_store_strlist(conf_h, conf->name,
-			&misc->searchdomain);
+			&misc->searchdomain, 0);
 		break;
 	}
 	return ret;
@@ -1255,6 +1254,8 @@ static int parse_cpulimit(unsigned long **param, const char *str)
 		return ERR_INVAL;
 
 	*param = malloc(sizeof(unsigned long));
+	if (!*param)
+		return ERR_NOMEM;
 	**param = val;
 	return 0;
 }
@@ -1262,39 +1263,21 @@ static int parse_cpulimit(unsigned long **param, const char *str)
 static int store_cpu(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
-	char buf[STR_SIZE];
 	cpu_param *cpu = &vps_p->res.cpu;
 
 	switch (conf->id) {
 	case PARAM_CPUUNITS:
-		if (cpu->units == NULL)
-			break;
-		snprintf(buf, sizeof(buf), "%s=\"%lu\"",
-				conf->name, *cpu->units);
-		add_str_param(conf_h, buf);
+		conf_store_ulong(conf_h, conf->name, cpu->units);
 		break;
 	case PARAM_CPUWEIGHT:
-		if (cpu->weight == NULL)
-			break;
-		snprintf(buf, sizeof(buf), "%s=\"%lu\"",
-			conf->name, *cpu->weight);
-		add_str_param(conf_h, buf);
+		conf_store_ulong(conf_h, conf->name, cpu->weight);
 		break;
 	case PARAM_CPULIMIT:
-		if (cpu->limit == NULL)
-			break;
-		snprintf(buf, sizeof(buf), "%s=\"%lu\"",
-			conf->name, *cpu->limit);
-		add_str_param(conf_h, buf);
+		conf_store_ulong(conf_h, conf->name, cpu->limit);
 		break;
 	case PARAM_VCPUS:
-		if (cpu->vcpus == NULL)
-			break;
-		snprintf(buf, sizeof(buf), "%s=\"%lu\"",
-			conf->name, *cpu->vcpus);
-		add_str_param(conf_h, buf);
+		conf_store_ulong(conf_h, conf->name, cpu->vcpus);
 		break;
-
 	}
 	return 0;
 }
@@ -1368,17 +1351,14 @@ static int parse_veth(vps_param *vps_p, char *val, int operation)
 	char *token;
 	veth_dev dev;
 
-	if ((token = strtok(val, " ")) == NULL)
-		return 0;
-	do {
+	for_each_strtok(token, val, " ") {
 		if (parse_veth_str(token, &dev, operation))
 			return ERR_INVAL;
 		if (operation)
 			ret = add_veth_param(&vps_p->res.veth, &dev);
 		else
 			ret = add_veth_param(&vps_p->del_res.veth, &dev);
-
-	} while ((token = strtok(NULL, " ")));
+	}
 
 	return 0;
 }
@@ -1472,7 +1452,8 @@ static int add_netif_param(veth_param *veth, int opt, char *str)
 static int parse_netif_str(envid_t veid, const char *str, veth_dev *dev)
 {
 	const char *p, *next, *ep;
-	int len, err;
+	unsigned int len;
+	int err;
 	char tmp[256];
 
 	memset(dev, 0, sizeof(*dev));
@@ -1561,21 +1542,15 @@ static int parse_netif(envid_t veid, veth_param *veth, char *val)
 	char *token;
 	veth_dev dev;
 
-	if ((token = strtok(val, ";")) == NULL)
-		return 0;
-	do {
+	for_each_strtok(token, val, ";") {
 		if (parse_netif_str(veid, token, &dev)) {
 			free_veth_dev(&dev);
 			continue;
 		}
-		if (find_veth_by_ifname_ve(&veth->dev,
-				dev.dev_name_ve) == NULL)
-		{
+		if (!find_veth_by_ifname_ve(&veth->dev, dev.dev_name_ve))
 			ret = add_veth_param(veth, &dev);
-		}
 		free_veth_dev(&dev);
-
-	} while ((token = strtok(NULL, ";")));
+	}
 
 	return 0;
 }
@@ -1713,10 +1688,10 @@ static int parse_netif_str_cmd(envid_t veid, const char *str, veth_dev *dev)
 		len = ch - tmp;
 		ch++;
 	}
-	
+
 	if (len) {
-	        if (len != MAC_SIZE) {
-		        logger(-1, 0, "Invalid container MAC address length");
+		if (len != MAC_SIZE) {
+			logger(-1, 0, "Invalid container MAC address length");
 			return ERR_INVAL;
 		}
 		err = parse_hwaddr(tmp, dev->dev_addr_ve);
@@ -1747,17 +1722,17 @@ static int parse_netif_str_cmd(envid_t veid, const char *str, veth_dev *dev)
 		ch++;
 	}
 	if (len) {
-	        if (len > IFNAMSIZE)
-		        return ERR_INVAL;
+		if (len > IFNAMSIZE)
+			return ERR_INVAL;
 		snprintf(dev->dev_name, len + 1, "%s", tmp);
 		if (ch == ep) {
-		        memcpy(dev->dev_addr, dev->dev_addr_ve, ETH_ALEN);
+			memcpy(dev->dev_addr, dev->dev_addr_ve, ETH_ALEN);
 			dev->addrlen = ETH_ALEN;
 			return 0;
 		}
 	} else {
-	        generate_veth_name(veid, dev->dev_name_ve, dev->dev_name,
-				   sizeof(dev->dev_name));
+		generate_veth_name(veid, dev->dev_name_ve, dev->dev_name,
+				sizeof(dev->dev_name));
 	}
 
 	tmp = ch;
@@ -1772,29 +1747,29 @@ static int parse_netif_str_cmd(envid_t veid, const char *str, veth_dev *dev)
 	}
 
 	if (len) {
-	        if (len != MAC_SIZE) {
-		        logger(-1, 0, "Invalid host MAC address");
+		if (len != MAC_SIZE) {
+			logger(-1, 0, "Invalid host MAC address");
 			return ERR_INVAL;
 		}
 		err = parse_hwaddr(tmp, dev->dev_addr);
 		if (err) {
-		  logger(-1, 0, "Invalid host MAC address");
-		  return ERR_INVAL;
+			logger(-1, 0, "Invalid host MAC address");
+			return ERR_INVAL;
 		}
 		dev->addrlen = ETH_ALEN;
 		if (ch == ep) {
-		        return 0;
+			return 0;
 		}
 	} else {
-	        generate_mac(veid, dev->dev_name, dev->dev_addr);
-	        dev->addrlen = ETH_ALEN;
+		generate_mac(veid, dev->dev_name, dev->dev_addr);
+		dev->addrlen = ETH_ALEN;
 	}
 
 	/* Parsing bridge name */
 	len = strlen (ch);
 
 	if (len > IFNAMSIZE)
-	        return ERR_INVAL;
+		return ERR_INVAL;
 
 	snprintf(dev->dev_bridge, len + 1, "%s", ch);
 
@@ -1807,14 +1782,11 @@ static int parse_netif_cmd(envid_t veid, veth_param *veth, char *val)
 	char *token;
 	veth_dev dev;
 
-	if ((token = strtok(val, " ")) == NULL)
-		return 0;
-	do {
+	for_each_strtok(token, val, " ") {
 		if (parse_netif_str_cmd(veid, token, &dev))
 			return ERR_INVAL;
 		ret = add_veth_param(veth, &dev);
-
-	} while ((token = strtok(NULL, " ")));
+	}
 
 	return 0;
 }
@@ -1843,47 +1815,49 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 {
 	int ret;
 	int int_id;
-	unsigned long uid;
 
 	ret = 0;
 	if (!_page_size) {
 		_page_size = get_pagesize();
-		if (_page_size == -1)
+		if (_page_size < 0)
 			return -1;
 	}
 	switch (id) {
 	case PARAM_CONFIG:
-		ret = conf_parse_str(&vps_p->opt.config, val, 1);
+		ret = conf_parse_str(&vps_p->opt.config, val);
 		break;
 	case PARAM_CONFIG_SAMPLE:
-		ret = conf_parse_str(&vps_p->opt.origin_sample, val, 1);
+		ret = conf_parse_str(&vps_p->opt.origin_sample, val);
 		break;
 	case PARAM_SAVE:
 		vps_p->opt.save = YES;
 		break;
+	case PARAM_FORCE:
+		vps_p->opt.save_force = YES;
+		break;
 	case PARAM_RESET_UB:
 		vps_p->opt.reset_ub = YES;
 		break;
 	case PARAM_ONBOOT:
-		ret = conf_parse_yesno(&vps_p->opt.onboot, val, 1);
+		ret = conf_parse_yesno(&vps_p->res.misc.onboot, val);
 		break;
 	case PARAM_DISABLED:
-		ret = conf_parse_yesno(&vps_p->opt.start_disabled, val, 1);
+		ret = conf_parse_yesno(&vps_p->opt.start_disabled, val);
 		break;
 	case PARAM_HOSTNAME:
-		ret = conf_parse_str(&vps_p->res.misc.hostname, val, 1);
+		ret = conf_parse_str(&vps_p->res.misc.hostname, val);
 		break;
 	case PARAM_DESCRIPTION:
-		ret = conf_parse_str(&vps_p->res.misc.description, val, 1);
+		ret = conf_parse_str(&vps_p->res.misc.description, val);
 		break;
 	case PARAM_NAMESERVER:
-		ret = conf_parse_strlist(&vps_p->res.misc.nameserver, val, 1);
+		ret = conf_parse_strlist(&vps_p->res.misc.nameserver, val);
 		break;
 	case PARAM_SEARCHDOMAIN:
-		ret = conf_parse_strlist(&vps_p->res.misc.searchdomain, val, 1);
+		ret = conf_parse_strlist(&vps_p->res.misc.searchdomain, val);
 		break;
 	case PARAM_USERPW:
-		ret = conf_parse_strlist(&vps_p->res.misc.userpw, val, 1);
+		ret = conf_parse_strlist(&vps_p->res.misc.userpw, val);
 		break;
 	case PARAM_APPLYCONFIG_MAP:
 		if (!strcmp(val, "name"))
@@ -1892,22 +1866,22 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 			ret = ERR_INVAL;
 		break;
 	case PARAM_APPLYCONFIG:
-		ret = conf_parse_str(&vps_p->opt.apply_cfg, val, 1);
+		ret = conf_parse_str(&vps_p->opt.apply_cfg, val);
 		break;
 	case PARAM_SETMODE:
 		ret = parse_setmode(vps_p, val);
 		break;
 	case PARAM_LOGFILE:
-		ret = conf_parse_str(&vps_p->log.log_file, val, 1);
+		ret = conf_parse_str(&vps_p->log.log_file, val);
 		break;
 	case PARAM_LOCKDIR:
-		ret = conf_parse_str(&vps_p->opt.lockdir, val, 1);
+		ret = conf_parse_str(&vps_p->opt.lockdir, val);
 		break;
 	case PARAM_DUMPDIR:
-		ret = conf_parse_str(&vps_p->res.cpt.dumpdir, val, 1);
+		ret = conf_parse_str(&vps_p->res.cpt.dumpdir, val);
 		break;
 	case PARAM_LOGGING:
-		ret = conf_parse_yesno(&vps_p->log.enable, val, 1);
+		ret = conf_parse_yesno(&vps_p->log.enable, val);
 		break;
 	case PARAM_LOGLEVEL:
 		if (parse_int(val, &int_id))
@@ -1933,6 +1907,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_PHYSPAGES:
 	case PARAM_VMGUARPAGES:
 	case PARAM_OOMGUARPAGES:
+	case PARAM_SWAPPAGES:
 		ret = parse_ub(vps_p, val, id, _page_size);
 		break;
 	case PARAM_NUMPROC:
@@ -1965,7 +1940,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		vps_p->res.net.skip_arpdetect = YES;
 		break;
 	case PARAM_IPV6NET:
-		ret = conf_parse_yesno(&vps_p->res.net.ipv6_net, val, 1);
+		ret = conf_parse_yesno(&vps_p->res.net.ipv6_net, val);
 		vps_p->del_res.net.ipv6_net = vps_p->res.net.ipv6_net;
 		break;
 	case PARAM_NETDEV_ADD:
@@ -1975,33 +1950,33 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		ret = add_netdev(&vps_p->del_res.net, val);
 		break;
 	case PARAM_ROOT:
-		if (!(ret = conf_parse_str(&vps_p->res.fs.root_orig, val, 1)))
+		if (!(ret = conf_parse_str(&vps_p->res.fs.root_orig, val)))
 			vps_p->res.fs.root = subst_VEID(veid, val);
 		break;
 	case PARAM_PRIVATE:
-		if (!(ret = conf_parse_str(&vps_p->res.fs.private_orig, val,1)))
+		if (!(ret = conf_parse_str(&vps_p->res.fs.private_orig, val)))
 			vps_p->res.fs.private = subst_VEID(veid, val);
 		break;
 	case PARAM_TEMPLATE:
-		ret = conf_parse_str(&vps_p->res.fs.tmpl, val, 1);
+		ret = conf_parse_str(&vps_p->res.fs.tmpl, val);
 		break;
 	case PARAM_NOATIME:
-		ret = conf_parse_yesno(&vps_p->res.fs.noatime, val, 1);
+		ret = conf_parse_yesno(&vps_p->res.fs.noatime, val);
 		break;
 	case PARAM_DEF_OSTEMPLATE:
-		ret = conf_parse_str(&vps_p->res.tmpl.def_ostmpl, val, 1);
+		ret = conf_parse_str(&vps_p->res.tmpl.def_ostmpl, val);
 		break;
 	case PARAM_PKGSET:
-		ret = conf_parse_str(&vps_p->res.tmpl.pkgset, val, 1);
+		ret = conf_parse_str(&vps_p->res.tmpl.pkgset, val);
 		break;
 	case PARAM_PKGVER:
-		ret = conf_parse_str(&vps_p->res.tmpl.pkgver, val, 1);
+		ret = conf_parse_str(&vps_p->res.tmpl.pkgver, val);
 		break;
 	case PARAM_OSTEMPLATE:
-		ret = conf_parse_str(&vps_p->res.tmpl.ostmpl, val, 1);
+		ret = conf_parse_str(&vps_p->res.tmpl.ostmpl, val);
 		break;
 	case PARAM_DISK_QUOTA:
-		ret = conf_parse_yesno(&vps_p->res.dq.enable, val, 1);
+		ret = conf_parse_yesno(&vps_p->res.dq.enable, val);
 		break;
 	case PARAM_DISKSPACE:
 		if (vps_p->res.dq.diskspace != NULL)
@@ -2016,24 +1991,10 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 			ret = ERR_INVAL;
 		break;
 	case PARAM_QUOTATIME:
-		if (vps_p->res.dq.exptime != NULL)
-			break;
-		vps_p->res.dq.exptime = malloc(sizeof(unsigned long));
-		if (parse_ul(val, vps_p->res.dq.exptime)) {
-			free(vps_p->res.dq.exptime);
-			vps_p->res.dq.exptime = NULL;
-			ret = ERR_INVAL;
-		}
+		ret = conf_parse_ulong(&vps_p->res.dq.exptime, val);
 		break;
 	case PARAM_QUOTAUGIDLIMIT:
-		if (vps_p->res.dq.ugidlimit != NULL)
-			break;
-		vps_p->res.dq.ugidlimit = malloc(sizeof(unsigned long));
-		if (parse_ul(val, vps_p->res.dq.ugidlimit)) {
-			free(vps_p->res.dq.ugidlimit);
-			vps_p->res.dq.ugidlimit = NULL;
-			ret = ERR_INVAL;
-		}
+		ret = conf_parse_ulong(&vps_p->res.dq.ugidlimit, val);
 		break;
 	case PARAM_DEVICES:
 		ret = parse_dev(vps_p, val);
@@ -2042,26 +2003,20 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		ret = parse_devnodes(vps_p, val);
 		break;
 	case PARAM_CPUUNITS:
-		if (vps_p->res.cpu.units != NULL)
+		ret = conf_parse_ulong(&vps_p->res.cpu.units, val);
+		if (ret != 0)
 			break;
-		if (parse_ul(val, &uid))
-			return ERR_INVAL;
-		if (uid < MINCPUUNITS || uid > MAXCPUUNITS)
-			return ERR_INVAL;
-		vps_p->res.cpu.units = malloc(sizeof(unsigned long));
-		if (vps_p->res.cpu.units == NULL)
-			return ERR_NOMEM;
-		*vps_p->res.cpu.units = uid;
+
+		if ((*vps_p->res.cpu.units < MINCPUUNITS ||
+				*vps_p->res.cpu.units > MAXCPUUNITS))
+		{
+			free(vps_p->res.cpu.units);
+			vps_p->res.cpu.units = NULL;
+			ret = ERR_INVAL;
+		}
 		break;
 	case PARAM_CPUWEIGHT:
-		if (vps_p->res.cpu.weight != NULL)
-			break;
-		if (parse_ul(val, &uid))
-			return ERR_INVAL;
-		vps_p->res.cpu.weight = malloc(sizeof(unsigned long));
-		if (vps_p->res.cpu.weight == NULL)
-			return ERR_NOMEM;
-		*vps_p->res.cpu.weight = uid;
+		ret = conf_parse_ulong(&vps_p->res.cpu.weight, val);
 		break;
 	case PARAM_CPULIMIT:
 		if (vps_p->res.cpu.limit != NULL)
@@ -2070,14 +2025,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 			return ERR_INVAL;
 		break;
 	case PARAM_VCPUS:
-		if (vps_p->res.cpu.vcpus != NULL)
-			break;
-		if (parse_ul(val, &uid))
-			return ERR_INVAL;
-		vps_p->res.cpu.vcpus = malloc(sizeof(unsigned long));
-		if (vps_p->res.cpu.vcpus == NULL)
-			return ERR_NOMEM;
-		*vps_p->res.cpu.vcpus = uid;
+		ret = conf_parse_ulong(&vps_p->res.cpu.vcpus, val);
 		break;
 	case PARAM_MEMINFO:
 		ret = parse_meminfo(&vps_p->res.meminfo, val);
@@ -2089,12 +2037,12 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		vps_p->res.veth.version = 1;
 		break;
 	case PARAM_VETH_ADD_CMD:
-		logger(0, 0, "Warning: --veth_add option is outdated use"
+		logger(0, 0, "Warning: --veth_add option is deprecated, use"
 			" --netif_add instead");
 		ret = parse_veth(vps_p, val, 1);
 		break;
 	case PARAM_VETH_DEL_CMD:
-		logger(0, 0, "Warning: --veth_del option is outdated use"
+		logger(0, 0, "Warning: --veth_del option is deprecated, use"
 			" --netif_del instead");
 		ret = parse_veth(vps_p, val, 0);
 		break;
@@ -2142,7 +2090,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 			break;
 		if (check_name(val))
 			return ERR_INVAL;
-		ret = conf_parse_str(&vps_p->res.name.name, val, 1);
+		ret = conf_parse_str(&vps_p->res.name.name, val);
 		break;
 	case PARAM_VEID:
 		if (parse_int(val, &int_id))
@@ -2156,6 +2104,9 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		if (parse_ioprio(id, &vps_p->res.io, val))
 			return ERR_INVAL;
 		break;
+	case PARAM_BOOTORDER:
+		ret = conf_parse_ulong(&vps_p->res.misc.bootorder, val);
+		break;
 	default:
 		logger(10, 0, "Not handled parameter %d %s", id, val);
 		break;
@@ -2216,8 +2167,10 @@ int vps_parse_config(envid_t veid, char *path, vps_param *vps_p,
 		str = malloc(len);
 	else
 		str = alloca(len);
-	if (str == NULL)
+	if (str == NULL) {
+		fclose(fp);
 		return VZ_RESOURCE_ERROR;
+	}
 	while (fgets(str, len, fp)) {
 		line++;
 		if ((rtoken = parse_line(str, ltoken, sizeof(ltoken))) == NULL)
@@ -2280,48 +2233,66 @@ static int read_conf(char *fname, list_head_t *conf_h)
 
 static int write_conf(char *fname, list_head_t *head)
 {
-	char buf[STR_SIZE];
+	char *tmpfile, *file;
 	conf_struct *conf;
-	int fd = 2;
-	int len, ret;
+	FILE * fp;
+	int ret = VZ_CONFIG_SAVE_ERROR;
+	const char *suffix = ".tmp";
+	char *fmt;
 
-	if (fname != NULL) {
-		snprintf(buf, sizeof(buf), "%s.tmp", fname);
-		if ((fd = open(buf, O_CREAT|O_WRONLY|O_TRUNC, 0644)) < 0) {
-			logger(-1, errno, "Unable to create configuration"
-				" file %s", buf);
+	file = canonicalize_file_name(fname);
+	if (file == NULL) {
+		if (errno != ENOENT)
+		{
+			logger(-1, errno, "Unable to resolve path %s", fname);
 			return 1;
 		}
+		file = strdup(fname);
+	}
+	tmpfile = vz_malloc(strlen(file) + strlen(suffix) + 1);
+	if (!tmpfile) {
+		goto out_file;
+	}
+	sprintf(tmpfile, "%s%s", file, suffix);
+	if ((fp = fopen(tmpfile, "w")) == NULL) {
+		logger(-1, errno, "Unable to create configuration"
+			" file %s", tmpfile);
+		goto out;
 	}
 	list_for_each(conf, head, list) {
 		if (conf->val == NULL)
 			continue;
-		len = strlen(conf->val);
-		ret = write(fd, conf->val, len);
-		if (ret < 0) {
-			logger(-1, errno, "Unable to write %d bytes to %s",
-					len, buf);
-			unlink(buf);
-			close(fd);
-			return 1;
-		}
 		if (strchr(conf->val, '\n') == NULL)
-			write(fd, "\n", 1);
-	}
-	if (fname != NULL) {
-		close(fd);
-		if (rename(buf, fname)) {
-			logger(-1, errno, "Unable to move %s -> %s",
-				buf, fname);
-			return 1;
+			fmt="%s\n";
+		else
+			fmt="%s";
+		if (fprintf(fp, fmt, conf->val) < 0) {
+			logger(-1, errno, "Error writing to %s",
+					tmpfile);
+			fclose(fp);
+			goto out2;
 		}
 	}
-	return 0;
+	fclose(fp);
+	if (rename(tmpfile, file)) {
+		logger(-1, errno, "Unable to move %s -> %s",
+			tmpfile, file );
+		goto out2;
+	}
+	ret = 0;
+out2:
+	unlink(tmpfile);
+out:
+	free(tmpfile);
+out_file:
+	free(file);
+	return ret;
 }
 
 static int vps_merge_conf(list_head_t *dst, list_head_t *src)
 {
-	int len, cnt = 0;
+	unsigned int len;
+	int cnt = 0;
 	conf_struct *conf, *line;
 	char ltoken[STR_SIZE];
 	char *p;
@@ -2365,12 +2336,12 @@ int vps_save_config(envid_t veid, char *path, vps_param *new_p,
 	if (action != NULL)
 		mod_save_config(action, &new_conf);
 	if ((ret = vps_merge_conf(&conf, &new_conf)) > 0)
-		write_conf(path, &conf);
+		ret = write_conf(path, &conf);
 	free_str_param(&conf);
 	free_str_param(&new_conf);
 	free_vps_param(tmp_old_p);
 
-	return 0;
+	return ret;
 }
 
 int vps_remove_cfg_param(envid_t veid, char *path, char *name)
@@ -2392,22 +2363,22 @@ int vps_remove_cfg_param(envid_t veid, char *path, char *name)
 		found++;
 	}
 	if (found)
-		write_conf(path, &conf);
+		ret = write_conf(path, &conf);
 	free_str_param(&conf);
 
-	return 0;
+	return ret;
 
 }
 
 /********************************************************************/
-int vps_parse_opt(envid_t veid, vps_param *param, int opt, char *rval,
-	struct mod_action *action)
+int vps_parse_opt(envid_t veid, struct option *opts, vps_param *param,
+	int opt, char *rval, struct mod_action *action)
 {
 	int id, ret = 0;
 
 	if (param == NULL)
 		return -1;
-	if ((id = opt_get_by_id(set_opt, opt)) != -1) {
+	if ((id = opt_get_by_id(opts, opt)) != -1) {
 		ret = parse(veid, param, rval, id);
 	} else if (action != NULL) {
 		ret = mod_parse(veid, action, NULL, opt, rval);
@@ -2447,10 +2418,10 @@ vps_param *init_vps_param()
 	return param;
 }
 
-int get_veid_by_name(char *name)
+int get_veid_by_name(const char *name)
 {
-	char buf[64];
-	char content[256];
+	char buf[STR_SIZE];
+	char content[STR_SIZE];
 	char *p;
 	int veid, ret;
 
@@ -2475,9 +2446,10 @@ int get_veid_by_name(char *name)
 static int check_name(char *name)
 {
 	char *p;
+	const char *validchars = " -+_.";
 
 	for (p = name; *p != '\0'; p++) {
-		if (!isdigit(*p) && !isalpha(*p) && *p != '-' && *p != '_')
+		if (!isdigit(*p) && !isalpha(*p) && !strchr(validchars, *p))
 			return -1;
 	}
 	return 0;
@@ -2528,7 +2500,7 @@ int set_name(int veid, char *new_name, char *old_name)
 	return 0;
 }
 
-#define FREE_P(x)	if ((x) != NULL) { free(x); x = NULL; }
+#define FREE_P(x)	free(x); x = NULL;
 
 static void free_opt(vps_opt *opt)
 {
@@ -2574,6 +2546,7 @@ static void free_misc(misc_param *misc)
 	free_str_param(&misc->userpw);
 	FREE_P(misc->hostname)
 	FREE_P(misc->description)
+	FREE_P(misc->bootorder)
 }
 
 static void free_net(net_param *net)
@@ -2654,31 +2627,30 @@ void free_vps_param(vps_param *param)
 #define MERGE_INT2(x)	if (src->x >= 0) dst->x = src->x;
 
 #define MERGE_P(x)							\
-	if ((src->x) != NULL) { 					\
-		if ((dst->x) == NULL) 					\
-			dst->x = malloc(sizeof(*(dst->x)));	 	\
+	if ((src->x) != NULL) {						\
+		if ((dst->x) == NULL)					\
+			dst->x = malloc(sizeof(*(dst->x)));		\
 		*dst->x = *src->x;					\
 	}
 
 #define MERGE_P2(x)							\
-	if ((src->x) != NULL) { 					\
-		if ((dst->x) == NULL) 					\
-			dst->x = malloc(sizeof(*(dst->x)) * 2);	 	\
+	if ((src->x) != NULL) {						\
+		if ((dst->x) == NULL)					\
+			dst->x = malloc(sizeof(*(dst->x)) * 2);		\
 		dst->x[0] = src->x[0];					\
 		dst->x[1] = src->x[1];					\
 	}
 
 static void merge_opt(vps_opt *dst, vps_opt *src)
 {
-	MERGE_INT(save);
-	MERGE_INT(fast_kill);
-	MERGE_INT(skip_lock);
-	MERGE_INT(skip_setup);
-	MERGE_INT(start_disabled);
-	MERGE_INT(start_force);
-	MERGE_INT(onboot);
-	MERGE_INT(setmode);
-	MERGE_INT(apply_cfg_map);
+	MERGE_INT(save)
+	MERGE_INT(fast_kill)
+	MERGE_INT(skip_lock)
+	MERGE_INT(skip_setup)
+	MERGE_INT(start_disabled)
+	MERGE_INT(start_force)
+	MERGE_INT(setmode)
+	MERGE_INT(apply_cfg_map)
 
 	MERGE_STR(config)
 	MERGE_STR(origin_sample)
@@ -2692,7 +2664,7 @@ static void merge_fs(fs_param *dst, fs_param *src)
 	MERGE_STR(private)
 	MERGE_STR(private_orig)
 	MERGE_STR(tmpl)
-	MERGE_INT(noatime);
+	MERGE_INT(noatime)
 }
 
 static void merge_tmpl(tmpl_param *dst, tmpl_param *src)
@@ -2733,7 +2705,9 @@ static void merge_misc(misc_param *dst, misc_param *src)
 	MERGE_LIST(userpw)
 	MERGE_STR(hostname)
 	MERGE_STR(description)
-	MERGE_INT(wait);
+	MERGE_INT(onboot)
+	MERGE_P(bootorder)
+	MERGE_INT(wait)
 }
 
 static void merge_dq(dq_param *dst, dq_param *src)
@@ -2760,31 +2734,31 @@ static void merge_dev(dev_param *dst, dev_param *src)
 
 static void merge_cap(cap_param *dst, cap_param *src)
 {
-	MERGE_INT(on);
-	MERGE_INT(off);
+	MERGE_INT(on)
+	MERGE_INT(off)
 }
 
 static void merge_env(env_param *dst, env_param *src)
 {
-	MERGE_INT(veid);
-	MERGE_INT(ipt_mask);
-	MERGE_INT(features_mask);
-	MERGE_INT(features_known);
+	MERGE_INT(veid)
+	MERGE_INT(ipt_mask)
+	MERGE_INT(features_mask)
+	MERGE_INT(features_known)
 }
 
 static void merge_cpt(cpt_param *dst, cpt_param *src)
 {
-	MERGE_STR(dumpdir);
-	MERGE_STR(dumpfile);
-	MERGE_INT(ctx);
-	MERGE_INT(cpu_flags);
-	MERGE_INT(cmd);
+	MERGE_STR(dumpdir)
+	MERGE_STR(dumpfile)
+	MERGE_INT(ctx)
+	MERGE_INT(cpu_flags)
+	MERGE_INT(cmd)
 }
 
 static void merge_meminfo(meminfo_param *dst, meminfo_param *src)
 {
-	MERGE_INT2(mode);
-	MERGE_INT(val);
+	MERGE_INT2(mode)
+	MERGE_INT(val)
 }
 
 static void merge_veth(veth_param *dst, veth_param *src)
@@ -2797,12 +2771,12 @@ static void merge_veth(veth_param *dst, veth_param *src)
 
 static void merge_name(name_param *dst, name_param *src)
 {
-	MERGE_STR(name);
-	MERGE_INT(veid);
+	MERGE_STR(name)
+	MERGE_INT(veid)
 }
 
 static void merge_io(io_param *dst, io_param *src) {
-	MERGE_INT2(ioprio);
+	MERGE_INT2(ioprio)
 }
 
 static int merge_res(vps_res *dst, vps_res *src)
@@ -2835,8 +2809,8 @@ int merge_vps_param(vps_param *dst, vps_param *src)
 
 int merge_global_param(vps_param *dst, vps_param *src)
 {
-	MERGE_INT(res.dq.enable);
-	MERGE_INT(res.net.ipv6_net);
-	MERGE_INT(del_res.net.ipv6_net);
+	MERGE_INT(res.dq.enable)
+	MERGE_INT(res.net.ipv6_net)
+	MERGE_INT(del_res.net.ipv6_net)
 	return 0;
 }
