@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,6 +16,10 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -29,6 +33,8 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <unistd.h>
+#include <dirent.h>
+
 
 #include "cpt.h"
 #include "env.h"
@@ -39,6 +45,8 @@
 #include "logger.h"
 #include "util.h"
 
+static int setup_hardlink_dir(const char *mntdir, int cpt_fd);
+
 int cpt_cmd(vps_handler *h, envid_t veid, int action, cpt_param *param,
 	vps_param *vps_p)
 {
@@ -47,7 +55,7 @@ int cpt_cmd(vps_handler *h, envid_t veid, int action, cpt_param *param,
 	const char *file;
 
 	if (!vps_is_run(h, veid)) {
-		logger(0, 0, "VE is not running");
+		logger(0, 0, "Container is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
 	if (action == CMD_CHKPNT) {
@@ -76,14 +84,15 @@ int cpt_cmd(vps_handler *h, envid_t veid, int action, cpt_param *param,
 	case CMD_KILL:
 		logger(0, 0, "Killing...");
 		if ((ret = ioctl(fd, CPT_KILL, 0)) < 0) {
-			logger(-1, errno, "Can not kill VE");
+			logger(-1, errno, "Can not kill container");
 			goto err;
 		}
 		break;
 	case CMD_RESUME:
 		logger(0, 0, "Resuming...");
+		clean_hardlink_dir(vps_p->res.fs.root);
 		if ((ret = ioctl(fd, CPT_RESUME, 0)) < 0) {
-			logger(-1, errno, "Can not resume VE");
+			logger(-1, errno, "Can not resume container");
 			goto err;
 		}
 		if (action == CMD_CHKPNT) {
@@ -106,7 +115,7 @@ err:
 	return ret ? err : 0;
 }
 
-int real_chkpnt(int cpt_fd, envid_t veid, char *root, cpt_param *param,
+int real_chkpnt(int cpt_fd, envid_t veid, const char *root, cpt_param *param,
 	int cmd)
 {
 	int ret, len, len1;
@@ -129,24 +138,29 @@ int real_chkpnt(int cpt_fd, envid_t veid, char *root, cpt_param *param,
 	if (cmd == CMD_CHKPNT || cmd == CMD_SUSPEND) {
 		logger(0, 0, "\tsuspend...");
 		if (ioctl(cpt_fd, CPT_SUSPEND, 0) < 0) {
-			logger(-1, errno, "Can not suspend VE");
+			logger(-1, errno, "Can not suspend container");
 			goto err_out;
 		}
 	}
 	if (cmd == CMD_CHKPNT || cmd == CMD_DUMP) {
 		logger(0, 0, "\tdump...");
+		clean_hardlink_dir("/");
+		if (setup_hardlink_dir("/", cpt_fd))
+			goto err_out;
 		if (ioctl(cpt_fd, CPT_DUMP, 0) < 0) {
-			logger(-1, errno, "Can not dump VE");
+			logger(-1, errno, "Can not dump container");
 			if (cmd == CMD_CHKPNT)
+				clean_hardlink_dir("/");
 				if (ioctl(cpt_fd, CPT_RESUME, 0) < 0)
-					logger(-1, errno, "Can not resume VE");
+					logger(-1, errno, "Can not "
+							"resume container");
 			goto err_out;
 		}
 	}
 	if (cmd == CMD_CHKPNT) {
 		logger(0, 0, "\tkill...");
 		if (ioctl(cpt_fd, CPT_KILL, 0) < 0) {
-			logger(-1, errno, "Can not kill VE");
+			logger(-1, errno, "Can not kill container");
 			goto err_out;
 		}
 	}
@@ -182,16 +196,16 @@ int vps_chkpnt(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 	int dump_fd = -1;
 	char dumpfile[PATH_LEN];
 	int cpt_fd, pid, ret;
-	char *root = vps_p->res.fs.root;
+	const char *root = vps_p->res.fs.root;
 
 	ret = VZ_CHKPNT_ERROR;
 	if (root == NULL) {
-		logger(-1, 0, "VE root is not set");
+		logger(-1, 0, "Container root (VE_ROOT) is not set");
 		return VZ_VE_ROOT_NOTSET;
 	}
 	if (!vps_is_run(h, veid)) {
 		logger(-1, 0, "Unable to setup checkpointing: "
-			"VE is not running");
+			"container is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
 	logger(0, 0, "Setting up checkpoint...");
@@ -210,9 +224,8 @@ int vps_chkpnt(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 					" specified.");
 				goto err;
 			}
-			snprintf(dumpfile, sizeof(dumpfile), "%s/"DEF_DUMPFILE,
-				vps_p->res.cpt.dumpdir != NULL ?
-				vps_p->res.cpt.dumpdir : DEF_DUMPDIR, veid);
+			get_dump_file(veid, vps_p->res.cpt.dumpdir,
+					dumpfile, sizeof(dumpfile));
 		}
 		dump_fd = open(param->dumpfile ? : dumpfile,
 			O_CREAT|O_TRUNC|O_RDWR, 0600);
@@ -230,7 +243,7 @@ int vps_chkpnt(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 		}
 	} else {
 		if (ioctl(cpt_fd, CPT_SET_VEID, veid) < 0) {
-			logger(0, errno, "Can not set veid");
+			logger(0, errno, "Can not set CT ID");
 			goto err;
 		}
 	}
@@ -241,9 +254,9 @@ int vps_chkpnt(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 		}
 	}
 	if (param->cpu_flags) {
-		logger(0, 0, "\tset cpu flags..");
+		logger(0, 0, "\tset CPU flags..");
 		if (ioctl(cpt_fd, CPT_SET_CPU_FLAGS, param->cpu_flags) < 0) {
-			logger(-1, errno, "Can not set cpu flags");
+			logger(-1, errno, "Can not set CPU flags");
 			goto err;
 		}
 	}
@@ -264,11 +277,13 @@ int vps_chkpnt(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 		ret = env_wait(pid);
 		exit(ret);
 	}
+	close(cpt_fd);
+	cpt_fd = -1;
 	ret = env_wait(pid);
 	if (ret)
 		goto err;
 	if (cmd == CMD_CHKPNT || cmd == CMD_DUMP) {
-		/* Clear VE network configuration */
+		/* Clear CT network configuration */
 		run_net_script(veid, DEL, &vps_p->res.net.ip, STATE_RUNNING,
 			vps_p->res.net.skip_arpdetect);
 		if (cmd == CMD_CHKPNT)
@@ -280,10 +295,13 @@ err:
 	if (ret) {
 		ret = VZ_CHKPNT_ERROR;
 		logger(-1, 0, "Checkpointing failed");
+		if (cmd == CMD_CHKPNT || cmd == CMD_DUMP)
+			unlink(param->dumpfile ? : dumpfile);
 	}
 	if (dump_fd != -1)
 		close(dump_fd);
-	close(cpt_fd);
+	if (cpt_fd != -1)
+		close(cpt_fd);
 
 	return ret;
 }
@@ -302,7 +320,7 @@ static int restrore_FN(vps_handler *h, envid_t veid, int wait_p, int err_p,
 	/* Close all fds */
 	close_fds(0, wait_p, err_p, h->vzfd, param->rst_fd, -1);
 	if (ioctl(param->rst_fd, CPT_SET_VEID, veid) < 0) {
-		logger(-1, errno, "Can't set VEID %d", param->rst_fd);
+		logger(-1, errno, "Can't set CT ID %d", param->rst_fd);
 		goto err;
 	}
 	if (pipe(error_pipe) < 0 ) {
@@ -328,14 +346,18 @@ static int restrore_FN(vps_handler *h, envid_t veid, int wait_p, int err_p,
 	* environment is created.
 	*/
 	close(STDIN_FILENO);
+
+	ioctl(param->rst_fd, CPT_HARDLNK_ON);
+
 	logger(0, 0, "\tundump...");
 	if (ioctl(param->rst_fd, CPT_UNDUMP, 0) < 0) {
 		logger(-1, errno, "Error: undump failed");
 		goto err_undump;
 	}
-	/* Now we wait until VE setup will be done */
+	/* Now we wait until CT setup will be done */
 	read(wait_p, &len, sizeof(len));
 	if (param->cmd == CMD_RESTORE) {
+		clean_hardlink_dir("/");
 		logger(0, 0, "\tresume...");
 		if (ioctl(param->rst_fd, CPT_RESUME, 0)) {
 			logger(-1, errno, "Error: resume failed");
@@ -377,10 +399,10 @@ int vps_restore(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 
 	if (vps_is_run(h, veid)) {
 		logger(-1, 0, "Unable to perform restore: "
-			"VE already running");
+			"container already running");
 		return VZ_VE_NOT_RUNNING;
 	}
-	logger(0, 0, "Restoring VE ...");
+	logger(0, 0, "Restoring container ...");
 	ret = VZ_RESTORE_ERROR;
 	if ((rst_fd = open(PROC_RST, O_RDWR)) < 0) {
 		if (errno == ENOENT)
@@ -402,10 +424,8 @@ int vps_restore(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 			goto err;
 		}
 
-		snprintf(dumpfile, sizeof(dumpfile), "%s/"DEF_DUMPFILE,
-			vps_p->res.cpt.dumpdir != NULL ?
-			vps_p->res.cpt.dumpdir : DEF_DUMPDIR,
-			veid);
+			get_dump_file(veid, vps_p->res.cpt.dumpdir,
+					dumpfile, sizeof(dumpfile));
 	}
 	if (cmd == CMD_RESTORE || cmd == CMD_UNDUMP) {
 		dump_fd = open(param->dumpfile ? : dumpfile, O_RDONLY);
@@ -427,16 +447,16 @@ int vps_restore(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 		NULL, restrore_FN, param);
 	if (ret)
 		goto err;
-	/* Restore 2level quota links & quota device */
+	/* Restore second-level quota links & quota device */
 	if ((cmd == CMD_RESTORE || cmd == CMD_UNDUMP) &&
 		vps_p->res.dq.ugidlimit != NULL && vps_p->res.dq.ugidlimit)
 	{
-		logger(0, 0, "Restore 2level quota");
+		logger(0, 0, "Restore second-level quota");
 		if (vps_execFn(h, veid, vps_p->res.fs.root, mk_quota_link, NULL,
 			VE_SKIPLOCK))
 		{
-			logger(-1, 0, "Warning: restore 2level quota links"
-				" failed");
+			logger(-1, 0, "Warning: restoring second-level "
+					"quota links failed");
 		}
 	}
 err:
@@ -448,3 +468,57 @@ err:
 	return ret;
 }
 
+/* with mix of md5sum: try generate unique name */
+#define CPT_HARDLINK_DIR ".cpt_hardlink_dir_a920e4ddc233afddc9fb53d26c392319"
+
+void clean_hardlink_dir(const char *mntdir)
+{
+	char buf[STR_SIZE];
+	struct dirent *ep;
+	DIR *dp;
+
+	snprintf(buf, sizeof(buf), "%s/%s", mntdir, CPT_HARDLINK_DIR);
+
+	unlink(buf);		/* if file was created by someone */
+	if (!(dp = opendir(buf)))
+		return;
+	while ((ep = readdir(dp))) {
+		if (!strcmp(ep->d_name, ".") || !strcmp(ep->d_name, ".."))
+			continue;
+
+		snprintf(buf, sizeof(buf), "%s/%s/%s",
+				mntdir, CPT_HARDLINK_DIR, ep->d_name);
+		unlink(buf);
+	}
+	closedir(dp);
+
+	snprintf(buf, sizeof(buf), "%s/%s", mntdir, CPT_HARDLINK_DIR);
+	rmdir(buf);
+}
+
+static int setup_hardlink_dir(const char *mntdir, int cpt_fd)
+{
+	char buf[STR_SIZE];
+	int fd, res = 0;
+
+	snprintf(buf, sizeof(buf), "%s/%s", mntdir, CPT_HARDLINK_DIR);
+	mkdir(buf, 0711);
+
+	fd = open(buf, O_RDONLY | O_NOFOLLOW | O_DIRECTORY);
+	if (fd < 0) {
+		logger(-1, errno, "Error: Unable to open "
+				"hardlink directory %s", buf);
+		return 1;
+	}
+
+	if (ioctl(cpt_fd, CPT_LINKDIR_ADD, fd) < 0) {
+		if (errno != EINVAL) {
+			res = 1;
+			logger(-1, errno, "Cannot set linkdir in kernel");
+		}
+		rmdir(buf);
+	}
+
+	close(fd);
+	return res;
+}
