@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -75,17 +75,18 @@ int vps_destroy_dir(envid_t veid, char *dir)
 	return 0;
 }
 
-static char *get_destroy_root(char *dir)
+static char *get_destroy_root(const char *dir)
 {
 	struct stat st;
-	int id, len;
-	char *p, *prev;
+	dev_t id;
+	int len;
+	const char *p, *prev;
 	char tmp[STR_SIZE];
 
 	if (stat(dir, &st) < 0)
 		return NULL;
 	id = st.st_dev;
-	p = dir + strlen(dir) - 1;
+	p = dir + strlen(dir);
 	prev = p;
 	while (p > dir) {
 		while (p > dir && (*p == '/' || *p == '.')) p--;
@@ -134,7 +135,7 @@ char *maketmpdir(const char *dir)
 /* Removes all the directories under 'root'
  * those names start with 'destroy_dir_magic'
  */
-static void _destroydir(char *root)
+static void _destroydir(const char *root)
 {
 	char buf[STR_SIZE];
 	struct stat st;
@@ -168,6 +169,16 @@ static void _destroydir(char *root)
 	} while(del);
 }
 
+static int _unlink(const char *s)
+{
+	if (unlink(s)) {
+		logger(-1, errno, "Unable to unlink %s", s);
+		return -1;
+	}
+	return 0;
+
+}
+
 static int destroydir(char *dir)
 {
 	char buf[STR_SIZE];
@@ -179,21 +190,36 @@ static int destroydir(char *dir)
 	int ret = 0;
 	struct stat st;
 
-	if (stat(dir, &st)) {
+	if (lstat(dir, &st)) {
 		if (errno != ENOENT) {
-			logger(-1, errno, "Unable to stat %s", dir);
+			logger(-1, errno, "Unable to lstat %s", dir);
 			return -1;
 		}
 		return 0;
 	}
+
+	if (S_ISLNK(st.st_mode)) {
+		int i;
+		i = readlink(dir, tmp, sizeof(tmp) - 1);
+		if (i == -1) {
+			logger(-1, 0, "Unable to readlink %s", dir);
+			return -1;
+		}
+		tmp[i] = '\0';
+		logger(-1, 0, "Warning: container private area %s "
+				"is a symlink to %s.\n"
+				"Not removing link destination, "
+				"you have to do it manually.",
+				dir, tmp);
+		return _unlink(dir);
+	}
+
 	if (!S_ISDIR(st.st_mode)) {
-		logger(-1, 0, "Warning: VE private area is not a directory");
-		if (unlink(dir)) {
-			logger(-1, errno, "Unable to unlink %s", dir);
-			return -1;
-		}
-		return 0;
+		logger(-1, 0, "Warning: container private area %s "
+				"is not a directory", dir);
+		return _unlink(dir);
 	}
+
 	root = get_destroy_root(dir);
 	if (root == NULL) {
 		logger(-1, 0, "Unable to get root for %s", dir);
@@ -233,7 +259,6 @@ static int destroydir(char *dir)
 	snprintf(buf, sizeof(buf), "%s/rm.lck", tmp);
 	if ((fd_lock = _lock(buf, 0)) == -2) {
 		/* Already locked */
-		_unlock(fd_lock, NULL);
 		return 0;
 	} else if (fd_lock == -1)
 		return VZ_FS_DEL_PRVT;
@@ -245,7 +270,7 @@ static int destroydir(char *dir)
 	sigaction(SIGCHLD, &act, NULL);
 
 	if (!(pid = fork())) {
-		int fd;
+		int fd, i;
 
 		setsid();
 		fd = open("/dev/null", O_WRONLY);
@@ -256,6 +281,10 @@ static int destroydir(char *dir)
 			dup2(fd, 1);
 			dup2(fd, 2);
 		}
+		for (i = 3; i < 1024; i++) {
+			if (i != fd_lock)
+				close(i);
+		}
 		_destroydir(tmp);
 		_unlock(fd_lock, buf);
 		exit(0);
@@ -277,20 +306,21 @@ int vps_destroy(vps_handler *h, envid_t veid, fs_param *fs)
 	if (check_var(fs->root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (vps_is_run(h, veid)) {
-		logger(0, 0, "VE is currently runing."
-			" Stop it before proceeding.");
+		logger(0, 0, "Container is currently running."
+			" Stop it first.");
 		return VZ_VE_RUNNING;
 	}
 	if (vps_is_mounted(fs->root)) {
-		logger(0, 0, "VE is currently mounted (umount first)");
+		logger(0, 0, "Container is currently mounted (umount first)");
 		return VZ_FS_MOUNTED;
 	}
-	logger(0, 0, "Destroying VE private area: %s", fs->private);
+	logger(0, 0, "Destroying container private area: %s", fs->private);
 	if ((ret = vps_destroy_dir(veid, fs->private)))
 		return ret;
 	move_config(veid, BACKUP);
-	rmdir(fs->root);
-	logger(0, 0, "VE private area was destroyed");
+	if (rmdir(fs->root) < 0)
+		logger(-1, errno, "Warning: failed to remove %s", fs->root);
+	logger(0, 0, "Container private area was destroyed");
 
 	return 0;
 }
