@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ static struct distr_conf {
 
 static int get_action_id(char *name)
 {
-	int i;
+	unsigned int i;
 
 	for (i = 0; i < sizeof(action2id) / sizeof(*action2id); i++)
 		if (!strcmp(name, action2id[i].name))
@@ -61,7 +61,7 @@ static int add_dist_action(dist_actions *d_actions, char *name, char *action,
 		return 0;
 	snprintf(file, sizeof(file), "%s/%s/%s", dir, DIST_SCRIPTS, action);
 	if (!stat_file(file)) {
-		logger(-1, 0, "Action script %s does not found", file);
+		logger(-1, 0, "Action script %s not found", file);
 		return 0;
 	}
 	switch (id) {
@@ -140,16 +140,19 @@ static int get_dist_conf_name(char *dist_name, char *dir, char *file, int len)
 			*ep = 0;
 		} while (ep > buf);
 		snprintf(file, len, "%s/%s", dir, DIST_CONF_DEF);
-		logger(-1, 0, "Warning: configuration file"
-			" for distribution %s not found default used",
-			dist_name);
+		logger(-1, 0, "Warning: configuration file "
+			"for distribution %s not found, "
+			"using defaults from %s/%s",
+			dist_name, dir, DIST_CONF_DEF);
 	} else {
 		snprintf(file, len, "%s/%s", dir, DIST_CONF_DEF);
-		logger(-1, 0, "Warning: distribution not specified"
-			" default used %s", file);
+		logger(-1, 0, "Warning: distribution not specified "
+			"in CT config, "
+			"using defaults from %s/%s", dir, DIST_CONF_DEF);
 	}
 	if (!stat_file(file)) {
-		logger(-1, 0, "Distribution configuration not found %s", file);
+		logger(-1, 0, "Distribution configuration file "
+			       "%s/%s not found", dir, file);
 		return VZ_NO_DISTR_CONF;
 	}
 	return 0;
@@ -160,7 +163,7 @@ static int get_dist_conf_name(char *dist_name, char *dir, char *file, int len)
  * @param dist_name	distribution name.
  * @param dir		directory distribution config file will be searched.
  * @param dist		filled dist_actions
- * @return 		0 on success
+ * @return		0 on success
  */
 int read_dist_actions(char *dist_name, char *dir, dist_actions *actions)
 {
