@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include "types.h"
 #include "logger.h"
 
-LOG_DATA
+log_param g_log = {NULL, 0, 1, 0, 0, "", 0};
 
 static inline void get_date(char *buf, int len)
 {
@@ -70,7 +70,7 @@ void logger(int log_level, int err_no, const char *format, ...)
 
 		fprintf(g_log.fp, "%s %s : ", date, g_log.prog);
 		if (g_log.veid)
-			fprintf(g_log.fp, "VE %d : ", g_log.veid);
+			fprintf(g_log.fp, "CT %d : ", g_log.veid);
 		vfprintf(g_log.fp, format, ap);
 		if (err_no)
 			fprintf(g_log.fp, ": %s", strerror(err_no));
@@ -114,7 +114,7 @@ void set_log_verbose(int level)
 }
 
 int init_log(char *file, envid_t veid, int enable, int level, int quiet,
-	char *progname)
+	const char *progname)
 {
 	int ret = 0;
 
@@ -127,7 +127,7 @@ int init_log(char *file, envid_t veid, int enable, int level, int quiet,
 	g_log.veid = veid;
 	g_log.quiet = quiet;
 	if (progname != NULL)
-		snprintf(g_log.prog, sizeof(g_log.prog), progname);
+		snprintf(g_log.prog, sizeof(g_log.prog), "%s", progname);
 	else
 		g_log.prog[0] = 0;
 
