@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 
 #define VZQUOTA		"/usr/sbin/vzquota"
 
-void quouta_inc(dq_param *param, int delta)
+void quota_inc(dq_param *param, int delta)
 {
 	if (param->enable != YES)
 		return;
@@ -143,7 +143,7 @@ int quota_init(envid_t veid, char *private, dq_param *param)
 	arg[i++] = strdup("-B");
 	snprintf(buf, sizeof(buf), "%lu", param->diskspace[1]);
 	arg[i++] = strdup(buf);
-	/*Disk inodes */
+	/* Disk inodes */
 	arg[i++] = strdup("-i");
 	snprintf(buf, sizeof(buf), "%lu", param->diskinodes[0]);
 	arg[i++] = strdup(buf);
@@ -218,7 +218,7 @@ int quota_on(envid_t veid, char *private, dq_param *param)
 	arg[i++] = strdup("-B");
 	snprintf(buf, sizeof(buf), "%lu", param->diskspace[1]);
 	arg[i++] = strdup(buf);
-	/*Disk inodes */
+	/* Disk inodes */
 	arg[i++] = strdup("-i");
 	snprintf(buf, sizeof(buf), "%lu", param->diskinodes[0]);
 	arg[i++] = strdup(buf);
@@ -300,10 +300,10 @@ int quota_off(envid_t veid, int force)
 	return ret;
 }
 
-/** Disk quota managment wraper.
+/** Disk quota management wrapper.
  *
  * @param veid		CT ID.
- * @param cmd		quota commands (QUOTA_MARKDURTY QUOTA_DROP QUOTA_STAT)
+ * @param cmd		quota commands (QUOTA_MARKDIRTY QUOTA_DROP QUOTA_STAT)
  * @return		0 on success.
  */
 int quota_ctl(envid_t veid, int cmd)
@@ -317,7 +317,7 @@ int quota_ctl(envid_t veid, int cmd)
 	i = 0;
 	arg[i++] = strdup(VZQUOTA);
 	switch (cmd) {
-	case QUOTA_MARKDURTY:
+	case QUOTA_MARKDIRTY:
 		arg[i++] =strdup( "setlimit");
 		snprintf(buf, sizeof(buf), "%d", veid);
 		arg[i++] = strdup(buf);
@@ -387,7 +387,7 @@ int vps_set_quota(envid_t veid, dq_param *dq)
 		return 0;
 	}
 	if (quota_ctl(veid, QUOTA_STAT)) {
-		logger(-1, 0, "Error: Unable to apply new quota values"
+		logger(-1, 0, "Error: Unable to apply new quota values:"
 			" quota not running");
 		return -1;
 	}
@@ -401,8 +401,8 @@ int vps_set_quota(envid_t veid, dq_param *dq)
 //			dq->ugidlimit = NULL;
 		} else if (!ret && !*dq->ugidlimit) {
 			logger(-1, 0, "WARNING: Unable to turn ugid quota"
-				" off. new parameters will be applyed"
-				" on next start");
+				" off. New parameters will be applied"
+				" during the next start");
 			tmp_ugidlimit = dq->ugidlimit;
 			dq->ugidlimit = NULL;
 		}
@@ -414,6 +414,13 @@ int vps_set_quota(envid_t veid, dq_param *dq)
 	return ret;
 }
 
+/** Turn disk quota on.
+ *
+ * @param veid		CT ID.
+ * @param private	CT private area path.
+ * @param dq		disk quota parameters.
+ * @return		0 on success.
+ */
 int vps_quotaon(envid_t veid, char *private, dq_param *dq)
 {
 	int ret;
@@ -421,7 +428,7 @@ int vps_quotaon(envid_t veid, char *private, dq_param *dq)
 	if (dq == NULL)
 		return 0;
 	if (dq->enable == NO) {
-//		quota_ctl(veid, QUOTA_MARKDURTY);
+//		quota_ctl(veid, QUOTA_MARKDIRTY);
 		return 0;
 	} else {
 		if (quota_ctl(veid, QUOTA_SHOW) == EXITCODE_QUOTNOTEXIST) {
@@ -435,6 +442,12 @@ int vps_quotaon(envid_t veid, char *private, dq_param *dq)
 	return 0;
 }
 
+/** Turn disk quota off.
+ *
+ * @param veid		CT ID.
+ * @param dq		disk quota parameters.
+ * @return		0 on success.
+ */
 int vps_quotaoff(envid_t veid, dq_param *dq)
 {
 	if (dq == NULL)
