@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -311,7 +311,8 @@ char *subst_VEID(envid_t veid, char *src)
 	char *srcp;
 	char str[STR_SIZE];
 	char *sp, *se;
-	int r, len, veidlen;
+	int r;
+	unsigned int len, veidlen;
 
 	if (src == NULL)
 		return NULL;
@@ -353,22 +354,22 @@ int get_pagesize()
 	long pagesize;
 
 	if ((pagesize = sysconf(_SC_PAGESIZE)) == -1) {
-		fprintf(stderr, "Unable to get page size");
-		return -11;
+		logger(-1, errno, "Unable to get page size");
+		return -1;
 	}
 	return pagesize;
 }
 
 int get_mem(unsigned long long *mem)
 {
-	long pagesize;
-	if ((*mem = sysconf(_SC_PHYS_PAGES)) == -1) {
+	long pages, pagesize;
+	if ((pages = sysconf(_SC_PHYS_PAGES)) == -1) {
 		logger(-1, errno, "Unable to get total phys pages");
 		return -1;
 	}
 	if ((pagesize = get_pagesize()) < 0)
 		return -1;
-	*mem *= pagesize;
+	*mem = pages * pagesize;
 	return 0;
 }
 
@@ -504,6 +505,12 @@ const char *get_vps_state_str(int vps_state)
 	return p;
 }
 
+int get_dump_file(unsigned veid, const char *dumpdir, char *buf, int size)
+{
+	return snprintf(buf, size, "%s/" DEF_DUMPFILE,
+			dumpdir != NULL ? dumpdir : DEF_DUMPDIR, veid);
+}
+
 int set_not_blk(int fd)
 {
 	int oldfl, ret;
@@ -522,7 +529,8 @@ int set_not_blk(int fd)
 */
 void close_fds(int close_std, ...)
 {
-	int fd, max, i;
+	int fd, max;
+	unsigned int i;
 	va_list ap;
 	int skip_fds[255];
 
@@ -540,7 +548,7 @@ void close_fds(int close_std, ...)
 	/* build aray of skiped fds */
 	va_start(ap, close_std);
 	skip_fds[0] = -1;
-	for (i = 0; i < sizeof(skip_fds); i++) {
+	for (i = 0; i < (sizeof(skip_fds)/sizeof(skip_fds[0])); i++) {
 		fd = va_arg(ap, int);
 		skip_fds[i] = fd;
 		if (fd == -1)
@@ -555,7 +563,7 @@ void close_fds(int close_std, ...)
 	}
 }
 
-/* Renames (to "*.destroyed" if action == MOVE) or removes config,
+/* Renames (to "*.destroyed" if action == BACKUP) or removes config,
  * (if action == DESTR)
  * Also, appropriate mount/umount scripts are linked.
  */
@@ -595,7 +603,8 @@ void remove_names(envid_t veid)
 	struct dirent *ep;
 	DIR *dp;
 	char *p;
-	int id;
+	int r;
+	envid_t id;
 
 	if (!(dp = opendir(VENAME_DIR)))
 		return;
@@ -605,10 +614,10 @@ void remove_names(envid_t veid)
 			continue;
 		if (!S_ISLNK(st.st_mode))
 			continue;
-		id = readlink(buf, content, sizeof(content) - 1);
-		if (id < 0)
+		r = readlink(buf, content, sizeof(content) - 1);
+		if (r < 0)
 			continue;
-		content[id] = 0;
+		content[r] = 0;
 		if ((p = strrchr(content, '/')) != NULL)
 			p++;
 		if (sscanf(p, "%d.conf", &id) == 1 && veid == id)
@@ -616,3 +625,23 @@ void remove_names(envid_t veid)
 	}
 	closedir(dp);
 }
+
+size_t vz_strlcat(char *dst, const char *src, size_t count)
+{
+	size_t dsize = strlen(dst);
+	size_t len = strlen(src);
+	size_t res = dsize + len;
+
+	if (dsize >= count)
+		return dsize;
+
+	dst += dsize;
+	count -= dsize;
+	if (len >= count)
+		len = count - 1;
+
+	memcpy(dst, src, len);
+	dst[len] = 0;
+
+	return res;
+}
