@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,11 @@ static struct feature_s features[] = {
 	{ "nfs",	0,	VE_FEATURE_NFS },
 	{ "sit",	0,	VE_FEATURE_SIT},
 	{ "ipip",	0,	VE_FEATURE_IPIP},
-	{ NULL}
+	{ "ppp",	0,	VE_FEATURE_PPP},
+	{ "ipgre",	0,	VE_FEATURE_IPGRE },
+	{ "bridge",	0,	VE_FEATURE_BRIDGE },
+	{ "nfsd",	0,	VE_FEATURE_NFSD },
+	{ NULL,		0,	0 }
 };
 
 struct feature_s *find_feature(const char *name)
@@ -37,16 +41,13 @@ struct feature_s *find_feature(const char *name)
 
 	for (feat = features; feat->name != NULL; feat++) {
 		len = strlen(feat->name);
-		if (strncmp(name, feat->name, len) == 0)
+		if (strncmp(name, feat->name, len) == 0 && name[len] == ':')
 			break;
 	}
 
 	if (feat->name == NULL)
 		return NULL;
 
-	if (name[len] != ':')
-		return NULL;
-
 	if (strcmp(name + len + 1, "on") == 0) {
 		feat->on = 1;
 		return feat;
