@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,11 +25,15 @@
 #include "config.h"
 #include "validate.h"
 #include "logger.h"
+#include "util.h"
+
+const char progname[] = "vzcfgvalidate";
+extern int page_size;
 
 void usage(int rc)
 {
 	FILE *fp = rc ? stderr : stdout;
-	fprintf(fp, "Usage: vzcfgvalidate [-r|-i] <configfile>\n");
+	fprintf(fp, "Usage: %s [-r|-i] <configfile>\n", progname);
 	fprintf(fp, "	-r repair mode\n");
 	fprintf(fp, "	-i interactive repair mode\n");
 	exit(rc);
@@ -37,13 +41,13 @@ void usage(int rc)
 
 int main(int argc, char **argv)
 {
-	vps_param *param;
+	vps_param *param, *gparam;
 	struct stat st;
 	char *infile = NULL;
-	int ret, opt, recover = 0, ask = 0;
+	int opt, recover = 0, ask = 0;
+	int ret = 1;
 
-	while ((opt = getopt(argc, argv, "rih")) > 0)
-	{
+	while ((opt = getopt(argc, argv, "rih")) > 0) {
 		switch(opt) {
 		case 'r'	:
 			recover = 1;
@@ -59,21 +63,46 @@ int main(int argc, char **argv)
 	}
 	if (optind >= argc)
 		usage(1);
+
+	init_log(NULL, 0, 1, 0, 0, progname);
+
+	if ((page_size = get_pagesize()) < 0)
+		return 1;
+
+	/* Read global config */
+	gparam = init_vps_param();
+	if (vps_parse_config(0, GLOBAL_CFG, gparam, NULL)) {
+		logger(-1, 0, "WARNING: Global configuration file %s "
+			"not found", GLOBAL_CFG);
+	}
+
+	/* Read container config */
 	infile = strdup(argv[optind]);
+	if (!infile) {
+		logger(-1, 0, "No free memory");
+		goto err;
+	}
 	if (stat(infile, &st)) {
-		fprintf(stderr,"VE configuration file: %s not found\n", infile);
-		free(infile);
-		exit(1);
+		logger(-1, 0, "Container configuration file %s not found",
+			infile);
+		goto err;
 	}
-	init_log(NULL, 0, 1, 0, 0, NULL);
 	param = init_vps_param();
 	if (vps_parse_config(0, infile, param, NULL))
-		exit(1);
-	if (!(ret = validate(&param->res, recover, ask)))	{
+		goto err;
+
+	/* Merge configs (needed for DISK_QUOTA value, maybe others) */
+	merge_vps_param(gparam, param);
+
+	if (!(ret = validate(&param->res, recover, ask))) {
 		if (recover || ask)
 			if (vps_save_config(0, infile, param, NULL, NULL))
-				return 1;
-		fprintf(stderr, "Validation completed: success\n");
+				goto err;
+		logger(-1, 0, "Validation completed: success");
 	}
+	ret = 0;
+
+err:
+	free(infile);
 	exit(ret);
 }
